/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: gfx_dif.c,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

/** @file gfx_dif.c
 *  gfx driver data interface layer
 *  
 */

//---------------------------------------------------------------------------
// Include files
//---------------------------------------------------------------------------

#include "gfx_if.h"
#include "gfx_drvif.h"
#include "gfx_dif.h"
#include "gfx_hw.h"
#include "gfx_sw.h"
#include "gfx_cmdque.h"
#include "gfx_debug.h"

//---------------------------------------------------------------------------
// Configurations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Type definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Macro definitions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported variables
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Imported functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static function forward declarations
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Static variables
//---------------------------------------------------------------------------

static MI_DIF_T _rDifData =
{ (UINT32)(GFX_HAVE_SW_MOD + GFX_HAVE_HW_MOD), (UINT32) E_GFX_HW_MOD,
        (UINT32) 0, (UINT32 *) NULL };

//---------------------------------------------------------------------------
// Static functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Inter-file functions
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------------


//-------------------------------------------------------------------------
/** pfnGFX_DifAction
 *  gfx command queue action function pointer
 */
//-------------------------------------------------------------------------
void (*pfnGFX_DifAction)(void);

//-------------------------------------------------------------------------
/** pfnGFX_DifFinNotify
 *  gfx complete notification function pointer
 */
//-------------------------------------------------------------------------
void (*pfnGFX_DifFinNotify)( UINT32);

//-------------------------------------------------------------------------
/** GFX_DifGetData
 *  get dif data (pointer)
 */
//-------------------------------------------------------------------------
MI_DIF_T *GFX_DifGetData(void)
{
    ASSERT_KERNEL();

    return &_rDifData;
}

//-------------------------------------------------------------------------
/** GFX_DifSetNotify
 *  set notification function
 */
//-------------------------------------------------------------------------
void GFX_DifSetNotify(void(*pfnNotify)( UINT32))
{
    ASSERT_KERNEL();

    pfnGFX_DifFinNotify = pfnNotify;
}

//-------------------------------------------------------------------------
/** GFX_DifInit
 *  gfx dif init
 *  init software module
 *  init hardware module
 *  set initial state (idle)
 */
//-------------------------------------------------------------------------
void GFX_DifInit(void)
{
    ASSERT_KERNEL();

#if defined(GFX_ENABLE_SW_MODE)
    //-----------------------------------------------------------
    GFX_SwInit();

    GFX_DifSetMode((UINT32)E_GFX_SW_MOD);

    MA_DIF_GFX_SW_MOD_OK();
    //-----------------------------------------------------------
#endif  //#if defined(GFX_ENABLE_SW_MODE)

    // setup ISR
    // reset hw and all counters
    // status check
    if (MA_DIF_HAVE_GFX_HW)
    {
        GFX_HwInit();

        MA_DIF_GFX_HW_MOD_OK();

        GFX_DifSetMode((UINT32) E_GFX_HW_MOD);
    }

    // fb module does not need initialization
    MA_DIF_GFX_FB_MOD_OK();
}

//-------------------------------------------------------------------------
/** GFX_DifSetRegBase
 *  gfx set dif register base address
 */
//-------------------------------------------------------------------------
void GFX_DifSetRegBase(UINT32 *pu4Base)
{
    ASSERT_KERNEL();

    GFX_ASSERT(pu4Base != NULL);

    _rDifData.pu4CrBase = pu4Base;
}

//-------------------------------------------------------------------------
/** GFX_DifGetRegBase
 *  gfx get dif register base address
 */
//-------------------------------------------------------------------------
void GFX_DifGetRegBase(UINT32 *pu4Base)
{
    ASSERT_KERNEL();

    GFX_ASSERT(pu4Base != NULL);

    *pu4Base = (UINT32)(_rDifData.pu4CrBase);
}

//-------------------------------------------------------------------------
/** GFX_DifReset
 *  gfx dif reset
 *  reset hardware module and software module
 */
//-------------------------------------------------------------------------
void GFX_DifReset(void)
{
    ASSERT_KERNEL();

    // if an HW gfx engine is available, just reset it
    if (MA_DIF_HAVE_GFX_HW)
    {
        GFX_HwReset();
    }
    else
    {
        // else erase software control register file
        GFX_UNUSED_RET(x_memset(_rDifData.pu4CrBase, 0,
                        (sizeof(UINT32) * GREG_FILE_SIZE)))
    }
}

//-------------------------------------------------------------------------
/** GFX_DifSetMode
 *  gfx set operation mode (hardware/software)
 *  change function pointers and re-init command queue
 */
//-------------------------------------------------------------------------
void GFX_DifSetMode(UINT32 u4GfxMode)
{
    UINT32 u4GfxRegBase;
    UINT32 *pu4GfxRegBase = &u4GfxRegBase;

    ASSERT_KERNEL();

    _rDifData.u4GfxMode = u4GfxMode;
    GFX_DBG_REC_ENG_EXE_MOD((INT32)u4GfxMode); // for debug use

    // hw mode setting
    if ((UINT32) E_GFX_HW_MOD == u4GfxMode)
    {
        pfnGFX_DifAction = GFX_HwAction;

        GFX_HwGetRegBase(&pu4GfxRegBase);

        GFX_DifSetRegBase(pu4GfxRegBase);
    }

#if defined(GFX_ENABLE_SW_MODE)
    // sw mode setting
    if ((UINT32)E_GFX_SW_MOD == u4GfxMode)
    {
        pfnGFX_DifAction = GFX_SwAction;

        GFX_SwGetRegBase(&pu4GfxRegBase);

        GFX_DifSetRegBase(pu4GfxRegBase);
    }

    // for emu-cli use
    GFX_CmdQueInit();
#endif  //#if defined(GFX_ENABLE_SW_MODE)
}
