/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * Description:
 *
 *---------------------------------------------------------------------------*/

#ifndef X_LIST_H
#define X_LIST_H

#include "x_typedef.h"

// List node
typedef struct _LIST_NODE_T
{
    struct _LIST_NODE_T*        pNext;
    struct _LIST_NODE_T*        pPrev;
    void*                       pObj;
} LIST_NODE_T;

typedef LIST_NODE_T*            POSITION;

// Function prototypes of memory allocation/free
typedef void* (*PFN_MEM_ALLOC)(UINT32);
typedef void (*PFN_MEM_FREE)(void*);

// The list
typedef struct
{
    POSITION                    pHead;
    POSITION                    pTail;
    UINT32                      u4Count;
    PFN_MEM_ALLOC               pfnAlloc;
    PFN_MEM_FREE                pfnFree;
} LIST_T;

//============================================================================
// Interface

extern BOOL ListInit(LIST_T* pList, PFN_MEM_ALLOC pfnAlloc,
    PFN_MEM_FREE pfnFree);
extern void ListDestroy(LIST_T* pList);
extern BOOL ListIsValidPosition(const LIST_T* pList, const POSITION pos);
extern BOOL ListAddHead(LIST_T* pList, void* pObj);
extern BOOL ListAddTail(LIST_T* pList, void* pObj);
extern BOOL ListAddAt(LIST_T* pList, void* pObj, const POSITION pos);
extern void* ListRemoveTail(LIST_T* pList);
extern void* ListRemoveHead(LIST_T* pList);
extern void* ListRemoveAt(LIST_T* pList, const POSITION pos);
extern BOOL ListRemoveAll(LIST_T* pList);
extern UINT32 ListGetCount(const LIST_T* pList);
extern BOOL ListIsEmpty(const LIST_T* pList);
extern POSITION ListGetHeadPosition(const LIST_T* pList);
extern POSITION ListGetTailPosition(const LIST_T* pList);
extern void* ListGetNext(const LIST_T* pList, POSITION* pPos);
extern void* ListGetPrev(const LIST_T* pList, POSITION* pPos);
extern void* ListGetAt(const LIST_T* pList, const POSITION pos);
extern INT32 ListGetIndex(const LIST_T* pList, const POSITION pos);
extern POSITION ListGetPosition(const LIST_T* pList, UINT32 u4Index);
extern POSITION ListFind(const LIST_T* pList, const void* pObj);

#endif	// X_LIST_H
