/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License 
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/
/*-----------------------------------------------------------------------------
 *
 * $Author: brianpc.huang $
 * $Date: 2015/05/29 $
 * $RCSfile: x_module.h,v $
 * $Revision: #1 $
 *
 *---------------------------------------------------------------------------*/

#ifndef X_MODULE_H
#define X_MODULE_H


//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

#include "x_typedef.h"
#include "x_os.h"
#include <linux/module.h>
#include <linux/init.h>


//---------------------------------------------------------------------------
// Constant definitions
//---------------------------------------------------------------------------

#ifdef MODULE_NODE_NAME

#ifdef MODULE_NODE_MAJOR
static int major = MODULE_NODE_MAJOR;
#else
static int major = 0;
#endif
module_param(major, int, 0);
#endif


//-----------------------------------------------------------------------------
// Macro definitions
//-----------------------------------------------------------------------------

static int __init _mod_init(unsigned long *_ioaddr)
{
    int result;
    UNUSED(result);

#if defined(MODULE_IO_ADDR) && defined(MODULE_IO_LEN)
    /* Get our needed resources. */
    if (! request_mem_region(MODULE_IO_ADDR, MODULE_IO_LEN,
        "MT5351 "MODULE_NODE_NAME))
    {
        printk(KERN_DEBUG MODULE_NODE_NAME
               ": can't get I/O mem address 0x%x\n", MODULE_IO_ADDR);
        return -ENODEV;
    }

    /* also, ioremap it */
    *_ioaddr = (unsigned long)ioremap(MODULE_IO_ADDR, MODULE_IO_LEN);
    /* Hmm... we should check the return value */
    printk(KERN_DEBUG MODULE_NODE_NAME"_ioaddr = 0x%lx\n", *_ioaddr);
#endif

#ifdef MODULE_NODE_NAME
    result = register_chrdev(major, MODULE_NODE_NAME, &MODULE_FOPS);
    if (result < 0)
    {
        printk(KERN_DEBUG MODULE_NODE_NAME": can't get major number\n");
        return result;
    }
    if (major == 0)
    {
        major = result;
    }
    printk(KERN_DEBUG MODULE_NODE_NAME" major = %d\n", major);
#endif

#ifdef MODULE_INIT_FUNC
    MODULE_INIT_FUNC();
#endif //MODULE_EXIT_FUNC

    return 0;
}

static void __exit _mod_exit(unsigned long _ioaddr)
{
#ifdef MODULE_NODE_NAME
    #ifdef MODULE_EXIT_FUNC
        MODULE_EXIT_FUNC();
    #endif //MODULE_EXIT_FUNC

    flush_scheduled_work();
    unregister_chrdev(major, MODULE_NODE_NAME);
#endif //MODULE_NODE_NAME

#if defined(MODULE_IO_ADDR) && defined(MODULE_IO_LEN)
    iounmap((void __iomem *)_ioaddr);
    release_mem_region(MODULE_IO_ADDR, MODULE_IO_LEN);
#endif
}


#define DECLARE_MODULE(mod)                                                 \
                                                                            \
unsigned long mod##_ioaddr = 0;                                             \
                                                                            \
static int __init mod##_init(void)                                          \
{                                                                           \
    _mod_init(&mod##_ioaddr);                                               \
                                                                            \
    return 0;                                                               \
}                                                                           \
                                                                            \
static void __exit mod##_exit(void)                                         \
{                                                                           \
    _mod_exit(mod##_ioaddr);                                                \
}                                                                           \
                                                                            \
module_init(mod##_init);                                                    \
module_exit(mod##_exit);                                                    \


#endif //X_MODULE_H
