#include <stdio.h>
#include <sys/mman.h>

#include <directfb.h>

#include <direct/debug.h>
#include <direct/messages.h>
#include <direct/system.h>

#include <core/gfxcard.h>
#include <core/system.h>

#include <mt53/mt53.h>

#include <gfx_if.h>

#include "mt53_2d.h"
#include "mt53_gfxdriver.h"

#include <core/graphics_driver.h>

DFB_GRAPHICS_DRIVER( mt53gfx )


/**********************************************************************************************************************/

static int
driver_probe( CoreGraphicsDevice *device )
{
     switch (dfb_gfxcard_get_accelerator( device )) {
          case FB_ACCEL_MEDIATEK_53XX:
               return 1;
     }

     return 0;
}

static void
driver_get_info( CoreGraphicsDevice *device,
                 GraphicsDriverInfo *info )
{
     /* fill driver info structure */
     snprintf( info->name,
               DFB_GRAPHICS_DRIVER_INFO_NAME_LENGTH,
               "MediaTek 53xx Driver" );

     snprintf( info->vendor,
               DFB_GRAPHICS_DRIVER_INFO_VENDOR_LENGTH,
               "Denis Oliver Kropp" );

     info->version.major = 0;
     info->version.minor = 3;

     info->driver_data_size = sizeof(MT53DriverData);
     info->device_data_size = sizeof(MT53DeviceData);
}

static DFBResult
driver_init_driver( CoreGraphicsDevice  *device,
                    GraphicsDeviceFuncs *funcs,
                    void                *driver_data,
                    void                *device_data,
                    CoreDFB             *core )
{
     MT53DriverData *mdrv = driver_data;
     MT53           *mt53 = dfb_system_data();

     mdrv->fd = mt53->fd;

     mdrv->shm_cmd_que = mmap( NULL, direct_pagesize(), PROT_READ | PROT_WRITE, MAP_SHARED,
                               mdrv->fd, direct_page_align( mt53->fix.smem_len ) );
     if (mdrv->shm_cmd_que == MAP_FAILED) {
          D_PERROR( "MediaTek/Driver: Mapping shared command queue control structure failed!\n" );
          return DFB_INIT;
     }

     D_INFO( "MediaTek/Driver: Mapped shared command queue control structure to %p\n", mdrv->shm_cmd_que );

     mdrv->cmd_que_buf = mmap( NULL, 256 * 1024, PROT_READ | PROT_WRITE, MAP_SHARED,
                               mdrv->fd, direct_page_align( mt53->fix.smem_len ) + direct_pagesize() );
     if (mdrv->cmd_que_buf == MAP_FAILED) {
          D_PERROR( "MediaTek/Driver: Mapping DMA region failed!\n" );
          munmap( (void*) mdrv->shm_cmd_que, direct_pagesize() );
          return DFB_INIT;
     }

     D_INFO( "MediaTek/Driver: Mapped DMA region to %p\n", mdrv->cmd_que_buf );

     GFX_CmdQueInit( mdrv->shm_cmd_que, &mdrv->cmd_que_buf );

     /* initialize function pointers */
     funcs->EngineSync   = mt53EngineSync;
     funcs->EngineReset  = mt53EngineReset;
     funcs->EmitCommands = mt53EmitCommands;
     funcs->CheckState   = mt53CheckState;
     funcs->SetState     = mt53SetState;

     return DFB_OK;
}

static DFBResult
driver_init_device( CoreGraphicsDevice *device,
                    GraphicsDeviceInfo *device_info,
                    void               *driver_data,
                    void               *device_data )
{
     /* fill device info */
     snprintf( device_info->vendor, DFB_GRAPHICS_DEVICE_INFO_VENDOR_LENGTH, "MediaTek" );
     snprintf( device_info->name,   DFB_GRAPHICS_DEVICE_INFO_NAME_LENGTH,   "53xx" );

     /* device limitations */
     device_info->limits.surface_byteoffset_alignment = 16;
     device_info->limits.surface_bytepitch_alignment  = 16;

     device_info->caps.flags    = 0;
     device_info->caps.accel    = MT53_SUPPORTED_DRAWINGFUNCTIONS |
                                  MT53_SUPPORTED_BLITTINGFUNCTIONS;
     device_info->caps.drawing  = MT53_SUPPORTED_DRAWINGFLAGS;
     device_info->caps.blitting = MT53_SUPPORTED_BLITTINGFLAGS;

     return DFB_OK;
}

static void
driver_close_device( CoreGraphicsDevice *device,
                     void               *driver_data,
                     void               *device_data )
{
}

static void
driver_close_driver( CoreGraphicsDevice *device,
                     void               *driver_data )
{
    MT53DriverData *mdrv = driver_data;

    munmap( (void*) mdrv->cmd_que_buf, 256 * 1024 );
    munmap( (void*) mdrv->shm_cmd_que, direct_pagesize() );
}

